/**
 * @file flowingttssdk.h
 * @author 挪麦 (lengjiayi.ljy@alibaba-inc.com)
 * @copyright Copyright (c) 2024 Alibaba
 * @date 2024-04-10
 * @brief :流式语音合成接口
 */
#ifndef COMMON_INCLUDE_STREAMINPUTTTSSDK_H_
#define COMMON_INCLUDE_STREAMINPUTTTSSDK_H_

enum StreamInputTtsEvent {
  STREAM_INPUT_TTS_EVENT_SYNTHESIS_STARTED = 0,
  STREAM_INPUT_TTS_EVENT_SENTENCE_BEGIN = 1,
  STREAM_INPUT_TTS_EVENT_SENTENCE_SYNTHESIS = 2,
  STREAM_INPUT_TTS_EVENT_SENTENCE_END = 3,
  STREAM_INPUT_TTS_EVENT_SYNTHESIS_COMPLETE = 4,
  STREAM_INPUT_TTS_EVENT_TASK_FAILED = 5,
};

typedef void (*FuncNuiStreamInputTtsListenerOnEvent)(
    void *user_data, char *taskid, char *session_id, StreamInputTtsEvent event,
    int code, char *err_message, int err_message_length, char *timestamp,
    int timestamp_length, char *all_response, int all_response_length);
typedef void (*FuncNuiStreamInputTtsListenerOnData)(void *user_data,
                                                    char *taskid,
                                                    char *session_id,
                                                    char *buffer, int len);

struct StreamInputTtsListener {
  FuncNuiStreamInputTtsListenerOnEvent stream_input_tts_event_callback;
  FuncNuiStreamInputTtsListenerOnData stream_input_tts_user_data_callback;
  void *user_data;
  StreamInputTtsListener() {
    stream_input_tts_event_callback = nullptr;
    stream_input_tts_user_data_callback = nullptr;
    user_data = nullptr;
  }
};

class StreamInputTtsSdk {
 public:
  StreamInputTtsSdk() { _request = nullptr; }
  ~StreamInputTtsSdk();
  int start(const char *ticket, const char *parameters, const char *session_id,
            StreamInputTtsListener *listener, int log_level, bool save_log);
  int send(const char *text);
  int stop();
  // int sendPing();

 private:
  // std::mutex mtx;
  void *_request;
};

#endif  // COMMON_INCLUDE_STREAMINPUTTTSSDK_H_